//preparation and calculations for data

void configInterrupt() {
  // Set intPin as interrupt, assign interrupt function updateCount() and set RISING mode
  attachInterrupt(digitalPinToInterrupt(intPin), ISR, RISING);
  /*There are 5 different modes:
    LOW: to trigger the interrupt whenever the pin is LOW;
    HIGH: to trigger the interrupt whenever the pin is HIGH;
    CHANGE: to trigger the interrupt whenever the pin changes value – for example from HIGH to LOW or LOW to HIGH;
    FALLING: for when the pin goes from HIGH to LOW;
    RISING: to trigger when the pin goes from LOW to HIGH.
  */
}

// detect and count up pulses
void IRAM_ATTR ISR() {
  // The IRAM_ATTR attribute places compiled code in the (faster) Internal RAM (IRAM) of the ESP32.
  tNow = millis();
  //max pulse rate is 1 per sec; 3600 watts in 3600 sec = 3.6kWh
  if (tNow - tLast > 300) { // prevent bounce
    pCount++; //increment count
    cWatts = pCount * 10; //adjusted (*10) as count is over 6 min not 60min
  }
  tLast = tNow;
}

void zeroStats() {  //zeroStats is called by setup and in newDay
  pCount = 0; //reset pulse count
  cWatts = 0; //reset current  watts
  mWatts = 0; //reset maximum watts
  pWatts = 0; //reset previous period
  dTotalkWh = 0; //reset running total for current day
}

//every 6 minutes:
void checkPeriod() {
  if (timeinfo.tm_min != cMin) { // new minute
    cMin = timeinfo.tm_min;
    if ((cMin % period) == 0) { // end of previous period
      pWatts = cWatts;
      if (cWatts > mWatts) {
        mWatts = cWatts; // update maximum value
      }
      //pCount holds the number of pulses received in the preceding (ie current) 6 min interval
      dTotalkWh += pCount / 1000.0;  //convert watts to kW & add to running total

      //change from version 2.06 to 2.10
      if (SPIFFS.exists(dayFileName)) {}  //checking for file for day data: if not present create one
      else { //no file ready
        Serial.println("checkPeriod: creating new day data file");
        //open a new SPIFFS file for 6 min data
        File dayFile = SPIFFS.open(dayFileName, FILE_WRITE);
        if (!dayFile) {
          Serial.println("checkPeriod: There was an error opening the file for writing");
          return;
        }
      } //end of new section

      //open dayFile for append, append pCount to dayFile, close dayFile
      appendInt(SPIFFS, dayFileName, pCount);
      Serial.print("checkPeriod: at time ");
      Serial.print(&timeinfo, "%H:%M:%S");  //eg 07:04:45
      Serial.printf("  cWatts: %4i; mWatts: %4i \n", cWatts, mWatts);
      pCount = 0; //reset pulse counter
      cWatts = 0; //else cWatts is invalid until first pulse in period is received
    }
  }
}

void newDay() {
  if (SPIFFS.exists(fileName)) {}  //checking for file for monthly data: filename created by newMonth() in setup
  else { //no file ready
    writeFile(SPIFFS, fileName, headerString); //char headerString[30] = "Day,total kWh,maxWatts \n";
    Serial.print("newDay: Filename is ");
    Serial.println(fileName);
  }

  //at day end read back day file and accumulate data
  float dTotalkWh = gatherDay(SPIFFS, dayFileName) / 1000.0;
  Serial.printf("\nnewDay: dTotalkWh is %6.3f\n", dTotalkWh);

  //format & save 1 line of data to monthly record
  snprintf(dataString, sizeof(dataString), "\n%3u, %6.3f, %3u", currDay, dTotalkWh, mWatts);
  appendFile(SPIFFS, fileName, dataString);
  //Serial.print("newDay: data string to file currDay, dTotalkWh, mWatts");
  //Serial.println(dataString);
  yTotalkWh = dTotalkWh; //store current total as "yesterdays total"

  //delete the existing day date file to start afresh
  deleteFile(SPIFFS, dayFileName);

  //open a new SPIFFS file for 6 min data
  File dayFile = SPIFFS.open(dayFileName, FILE_WRITE);
  if (!dayFile) {
    Serial.println("newDay: There was an error opening the file for writing");
    return;
  }
  zeroStats();
}

void newMonth() {
  //create new filename for saving data and append header line; handle new day first; filename needs leading /
  strftime (fileName, sizeof(fileName), "/%Y_%m.csv", &timeinfo); // "/" + 4 digit year + "_" + 2 digit month +".csv" + null = 12 chars
  //nb strftime on timeinfo gives correct month value eg 05 for May
  if (SPIFFS.exists(fileName)) {}
  else { //no file ready
    writeFile(SPIFFS, fileName, headerString); //char headerString[30] = "Day,total kWh,maxWatts \n";
    Serial.print("newMonth: Filename is ");
    Serial.println(fileName);
  }
  currMonth = timeinfo.tm_mon; //update current month
  Serial.print("newMonth: file header is ");
  Serial.println(headerString);
}
